﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.TestFunctions {
  /// <summary>
  /// An operator that performs similarity calculation between two test functions solutions.
  /// </summary>
  [Item("SingleObjectiveTestFunctionSimilarityCalculator", "An operator that performs similarity calculation between two test functions solutions.")]
  [StorableClass]
  public sealed class SingleObjectiveTestFunctionSimilarityCalculator : SimilarityCalculator {
    #region Properties
    public DoubleMatrix Bounds { get; set; }
    #endregion

    [StorableConstructor]
    private SingleObjectiveTestFunctionSimilarityCalculator(bool deserializing) : base(deserializing) { }
    private SingleObjectiveTestFunctionSimilarityCalculator(SingleObjectiveTestFunctionSimilarityCalculator original, Cloner cloner) : base(original, cloner) { }
    public SingleObjectiveTestFunctionSimilarityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleObjectiveTestFunctionSimilarityCalculator(this, cloner);
    }

    protected override double CalculateSimilarity(IScope left, IScope right) {
      RealVector sol1 = left.Variables["Point"].Value as RealVector;
      RealVector sol2 = right.Variables["Point"].Value as RealVector;

      double maxSum = 0.0;
      for (int i = 0; i < sol1.Length; i++)
        maxSum += Math.Pow(Bounds[0, 0] - Bounds[0, 1], 2);
      double maxDistance = Math.Sqrt(maxSum) / sol1.Length;

      double sum = 0.0;
      for (int i = 0; i < sol1.Length; i++)
        sum += Math.Pow(sol1[i] - sol2[i], 2);
      double distance = Math.Sqrt(sum) / sol1.Length;

      return 1.0 - distance / maxDistance;
    }
  }
}
